@@ -3,12 +3,12 @@ np = sensenet.importers.import_numpy()
 tf = sensenet.importers.import_tensorflow()
 kl = sensenet.importers.import_keras_layers()
 
-from sensenet.constants import MAX_BOUNDING_BOXES, MAX_OBJECTS
+from sensenet.constants import MAX_OBJECTS
 from sensenet.constants import SCORE_THRESHOLD, IGNORE_THRESHOLD, IOU_THRESHOLD
 from sensenet.accessors import number_of_classes, get_image_shape
 from sensenet.layers.yolo import YoloTrunk, YoloBranches
 from sensenet.models.settings import ensure_settings
-from sensenet.preprocess.image import ImageReader, ImageLoader
+from sensenet.preprocess.image import BoundingBoxImageReader, ImageLoader
 from sensenet.pretrained import load_pretrained_weights
 
 class BoxLocator(tf.keras.layers.Layer):
@@ -22,7 +22,7 @@ class BoxLocator(tf.keras.layers.Layer):
         self._iou_threshold = settings.iou_threshold or IOU_THRESHOLD
         self._max_objects = settings.max_objects or MAX_OBJECTS
 
-    def filter_boxes(self, box_xywh, scores):
+    def filter_boxes(self, box_xywh, scores, limits):
         scores_max = tf.math.reduce_max(scores, axis=-1)
         mask = scores_max >= min(0.2, self._threshold)
 
@@ -49,12 +49,18 @@ class BoxLocator(tf.keras.layers.Layer):
             box_maxes[..., 1:2]  # x_max
         ], axis=-1)
 
+        ylim, xlim = limits[0], limits[1]
+
+        amask = tf.logical_and(box_xy[..., 0] < xlim, box_xy[..., 1] < ylim)
+        boxes = tf.reshape(tf.boolean_mask(boxes, amask), boxes_shape)
+        pred_conf = tf.reshape(tf.boolean_mask(pred_conf, amask), conf_shape)
+
         return boxes, pred_conf
 
     def reshape3d(self, x):
         return tf.reshape(x, (tf.shape(x)[0], -1, tf.shape(x)[-1]))
 
-    def call(self, predictions):
+    def call(self, predictions, original_shape):
         box_bounds = []
         box_scores = []
 
@@ -68,7 +74,13 @@ class BoxLocator(tf.keras.layers.Layer):
         boxes = tf.concat(box_bounds, axis=1)
         scores = tf.concat(box_scores, axis=1)
 
-        boxes, scores = self.filter_boxes(boxes, scores)
+        # Here we're assuming that we only get one image at a time as input
+        # I'm not sure this can be vectorized, given the flattening that
+        # happens when we mask above
+        max_dim = tf.reduce_max(original_shape[0][:2])
+        limits = self._input_shape * original_shape[0][:2] / max_dim
+
+        boxes, scores = self.filter_boxes(boxes, scores, limits)
         sc_shape = tf.shape(scores)
 
         boxes, scores, classes, valid = tf.image.combined_non_max_suppression(
@@ -81,7 +93,7 @@ class BoxLocator(tf.keras.layers.Layer):
         )
 
         vboxes = tf.gather(boxes[0,:valid[0],...], [1,0,3,2], axis=-1)
-        vboxes = tf.math.round(vboxes * self._input_shape[0])
+        vboxes = tf.math.round(vboxes * max_dim)
 
         return vboxes, scores[0,:valid[0]], classes[0,:valid[0]]
 
@@ -89,15 +101,15 @@ def box_detector(model, input_settings):
     settings = ensure_settings(input_settings)
 
     network = model['image_network']
-    reader = ImageReader(network, settings)
+    reader = BoundingBoxImageReader(network, settings)
 
     if settings.input_image_format == 'pixel_values':
         image_shape = get_image_shape(model)
         image_input = kl.Input(image_shape[1:], dtype=tf.float32, name='image')
-        raw_image = reader(image_input)
+        raw_image, original_shape = reader(image_input)
     else:
         image_input = kl.Input((1,), dtype=tf.string, name='image')
-        raw_image = reader(image_input[:,0])
+        raw_image, original_shape = reader(image_input[:,0])
 
     nclasses = number_of_classes(model)
     loader = ImageLoader(network)
@@ -109,6 +121,6 @@ def box_detector(model, input_settings):
     image = loader(raw_image)
     layer_outputs = yolo_trunk(image)
     predictions = yolo_branches(layer_outputs)
-    all_outputs = locator(predictions)
+    all_outputs = locator(predictions, tf.cast(original_shape, tf.float32))
 
     return tf.keras.Model(inputs=image_input, outputs=all_outputs)
